@@ -62,8 +62,63 @@ func CheckDatabaseConnectivity(hostPortList string) error {
 	return fmt.Errorf("connectivity cannot be established with any endpoint: %v", lastErr)
 }
 
+// importCustomCA imports the custom S3 CA certificate into a JVM truststore if S3_CA_CERT_PATH is set.
+func importCustomCA() error {
+	caPath := os.Getenv("S3_CA_CERT_PATH")
+	if caPath == "" {
+		log.Println("No custom S3 CA certificate path set, skipping truststore import.")
+		return nil
+	}
+
+	truststorePath := "/tmp/custom-truststore.jks"
+	truststorePass := os.Getenv("TRUSTSTORE_PASSWORD")
+	if truststorePass == "" {
+		truststorePass = "changeit" // Default for backward compatibility
+	}
+
+	// Check if keytool is available
+	if _, err := exec.LookPath("keytool"); err != nil {
+		return fmt.Errorf("keytool not found in PATH: %v", err)
+	}
+
+	// Import the CA certificate into the truststore
+	cmd := exec.Command("keytool",
+		"-importcert",
+		"-noprompt",
+		"-trustcacerts",
+		"-alias", "custom-s3-ca",
+		"-file", caPath,
+		"-keystore", truststorePath,
+		"-storepass", truststorePass,
+	)
+
+	output, err := cmd.CombinedOutput()
+	if err != nil {
+		return fmt.Errorf("failed to import CA certificate: %v\noutput: %s", err, string(output))
+	}
+
+	log.Printf("Successfully imported CA certificate from %s into truststore %s", caPath, truststorePath)
+
+	// Set JAVA_OPTS to use the custom truststore
+	javaOpts := fmt.Sprintf("-Djavax.net.ssl.trustStore=%s -Djavax.net.ssl.trustStorePassword=%s", truststorePath, truststorePass)
+	if existingOpts := os.Getenv("JAVA_OPTS"); existingOpts != "" {
+		javaOpts = existingOpts + " " + javaOpts
+	}
+	if err := os.Setenv("JAVA_OPTS", javaOpts); err != nil {
+		return fmt.Errorf("failed to set JAVA_OPTS: %v", err)
+	}
+
+	log.Printf("Set JAVA_OPTS: %s", javaOpts)
+	return nil
+}
+
 // PerformBackup performs the backup operation and returns the generated backup file name
 func PerformBackup(address string) ([]string, error) {
+	// Import custom CA if needed
+	if err := importCustomCA(); err != nil {
+		return nil, fmt.Errorf("failed to import custom CA: %v", err)
+	}
+
 	databases := strings.ReplaceAll(os.Getenv("DATABASE"), ",", " ")
 	flags := getBackupCommandFlags(address)
 	log.Printf("Printing backup flags %v", flags)
@@ -186,13 +241,42 @@ func PerformConsistencyCheck(database string, backupFileName string) (string, er
 	log.Printf("Backup file name: %s", fileName)
 	log.Printf("Cloud provider: %s", cloudProvider)
 
-	// Increase timeout to 30 minutes for cloud storage consistency checks
-	timeout := 30 * time.Minute
-	if cloudProvider != "" {
-		log.Printf("Using extended timeout of %v for cloud storage consistency check", timeout)
-	}
+	// Set timeout for consistency checks - configurable via CONSISTENCY_CHECK_TIMEOUT environment variable
+	// Empty string means no timeout (for local storage)
+	var ctx context.Context
+	var cancel context.CancelFunc
+	var timeoutDuration time.Duration
 
-	ctx, cancel := context.WithTimeout(context.Background(), timeout)
+	timeoutEnv := os.Getenv("CONSISTENCY_CHECK_TIMEOUT")
+	if timeoutEnv != "" {
+		// Parse and apply timeout
+		if parsedTimeout, err := time.ParseDuration(timeoutEnv); err != nil {
+			// Invalid format - use default 30m for cloud, no timeout for local
+			if cloudProvider != "" {
+				timeoutDuration = 30 * time.Minute
+				log.Printf("Warning: Invalid timeout format '%s', using default 30m: %v", timeoutEnv, err)
+				ctx, cancel = context.WithTimeout(context.Background(), timeoutDuration)
+			} else {
+				log.Printf("Warning: Invalid timeout format '%s', no timeout applied for local storage: %v", timeoutEnv, err)
+				ctx, cancel = context.WithCancel(context.Background())
+			}
+		} else {
+			// Valid timeout provided
+			timeoutDuration = parsedTimeout
+			log.Printf("Using configured timeout of %v for consistency check", timeoutDuration)
+			ctx, cancel = context.WithTimeout(context.Background(), timeoutDuration)
+		}
+	} else {
+		// No timeout specified - use default for cloud, none for local
+		if cloudProvider != "" {
+			timeoutDuration = 30 * time.Minute
+			log.Printf("Using default extended timeout of %v for cloud storage consistency check", timeoutDuration)
+			ctx, cancel = context.WithTimeout(context.Background(), timeoutDuration)
+		} else {
+			log.Printf("No timeout applied for local storage consistency check")
+			ctx, cancel = context.WithCancel(context.Background())
+		}
+	}
 	defer cancel()
 
 	cmd := exec.CommandContext(ctx, "neo4j-admin", flags...)
@@ -259,7 +343,7 @@ func PerformConsistencyCheck(database string, backupFileName string) (string, er
 	log.Printf("Consistency check output: %s", outputBuffer.String())
 
 	if ctx.Err() == context.DeadlineExceeded {
-		return "", fmt.Errorf("Consistency check timed out after %v for database %s", timeout, database)
+		return "", fmt.Errorf("Consistency check timed out after %v for database %s", timeoutDuration, database)
 	}
 
 	if err == nil {
@@ -323,7 +407,30 @@ func countLocalBackups(path, db string) (int, error) {
 }
 
 func countS3Backups(db, bucket, pathPrefix string) (int, error) {
-	sess, err := session.NewSession(&aws.Config{})
+	config := &aws.Config{}
+
+	// Check for custom S3 endpoint from environment variable
+	if endpoint := os.Getenv("AWS_ENDPOINT_URL_S3"); endpoint != "" {
+		config.Endpoint = aws.String(endpoint)
+		log.Printf("Using custom S3 endpoint: %s", endpoint)
+	}
+
+	// Set S3 force path style if configured
+	if forcePathStyle := os.Getenv("S3_FORCE_PATH_STYLE"); forcePathStyle == "true" {
+		config.S3ForcePathStyle = aws.Bool(true)
+		log.Printf("Using S3 force path style")
+	}
+
+	// Set region if configured
+	if region := os.Getenv("AWS_REGION"); region != "" {
+		config.Region = aws.String(region)
+		log.Printf("Using AWS region: %s", region)
+	} else if region := os.Getenv("AWS_DEFAULT_REGION"); region != "" {
+		config.Region = aws.String(region)
+		log.Printf("Using AWS default region: %s", region)
+	}
+
+	sess, err := session.NewSession(config)
 	if err != nil {
 		return 0, err
 	}
